import requests
import json
from metromobilite.metromobilite_exception import MetromobiliteRequestException, MetromobiliteInvalidDataException

TIMEOUT = 10

class Metromobilite:
    API_BASE_URL = 'http://data.metromobilite.fr/api/'

    def __init__(self, origin):
        self.origin = origin


    def make_request(self, url, params=None):
        """Make the metromobilite API request"""

        headers = {'origin': self.origin}
        response = requests.get(url, headers=headers, timeout=TIMEOUT, params=params)

        if response.status_code != 200:
            raise MetromobiliteRequestException(response.status_code)

        return self.parse_json(response.content)

    def parse_json(self, content):
        try:
            return json.loads(content)
        except ValueError:
            raise MetromobiliteInvalidDataException()

    def get_stoptimes(self, stop_id):
        url = self.API_BASE_URL + 'routers/default/index/stops/' + stop_id + '/stoptimes'

        return self.make_request(url)

    def get_lines_near(self, latitude, longitude, dist, details):
        url = self.API_BASE_URL + 'api/linesNear/json'
        params = {'latitude': latitude, 'longitude': longitude, 'dist': dist, 'details': details}
        return self.make_request(url, params)



class MetromobiliteException(Exception):
    pass

class MetromobiliteInvalidDataException(Exception):
    def __init__(self):
        super(MetromobiliteInvalidDataException, self).__init__("Error while parsing json")


class MetromobiliteRequestException(MetromobiliteException):
     def __init__(self, status_code):
        super(MetromobiliteRequestException, self).__init__("Http Error : " + str(status_code))



import pytest
import requests_mock
from metromobilite import Metromobilite
from metromobilite.metromobilite_exception import MetromobiliteInvalidDataException, MetromobiliteRequestException



class TestMetromobilite:
    def setup_class(self):
        self.m = Metromobilite(origin='Test request')

    def test_make_request_exception(self):
        with requests_mock.Mocker() as r:
            r.get("https://example.com", status_code=404)
            with pytest.raises(MetromobiliteRequestException):
                self.m.make_request("https://example.com")
    
    def test_parse_json(self):
        json = """{"test": "test"}"""
        content = self.m.parse_json(json)
        assert {'test': 'test'} == content


    def test_parse_json_exception(self):
        with pytest.raises(MetromobiliteInvalidDataException):
            self.m.parse_json("")

    def test_get_stoptimes(self):
         with requests_mock.Mocker() as r:
            stop_id = "test"
            r.get(self.m.API_BASE_URL + 'routers/default/index/stops/' + stop_id + '/stoptimes' , json="""{"test": "test"}""")
            response = self.m.get_stoptimes(stop_id)
            assert """{"test": "test"}""" == response

    def test_get_lines_near(self):
        with requests_mock.Mocker() as r:
            r.get(self.m.API_BASE_URL + 'api/linesNear/json', json="""{"test": "test"}""")
            response = self.m.get_lines_near(latitude='test', longitude='test', dist='test', details='test')
            assert """{"test": "test"}""" == response

